@@ -267,6 +267,9 @@ secret_item_finalize (GObject *obj)
 		g_object_remove_weak_pointer (G_OBJECT (self->pv->service),
 		                              (gpointer *)&self->pv->service);
 
+	if (self->pv->value != NULL)
+		secret_value_unref (self->pv->value);
+
 	g_mutex_clear (&self->pv->mutex);
 
 	G_OBJECT_CLASS (secret_item_parent_class)->finalize (obj);
@@ -687,7 +690,7 @@ on_create_item (GObject *source,
 	if (error != NULL)
 		g_simple_async_result_take_error (res, error);
 
-	/* As a convenince mark down the SecretValue on the item */
+	/* As a convenience mark down the SecretValue on the item */
 	_secret_item_set_cached_secret (closure->item, closure->value);
 
 	g_simple_async_result_complete (res);
@@ -714,6 +717,7 @@ on_create_path (GObject *source,
 		g_simple_async_result_take_error (res, error);
 		g_simple_async_result_complete (res);
 	}
+	g_free (path);
 
 	g_object_unref (res);
 }
