@@ -149,7 +149,7 @@ G_DEFINE_BOXED_TYPE (SecretSchemaAttribute, secret_schema_attribute,
                      schema_attribute_copy, schema_attribute_free);
 
 /**
- * secret_schema_newv:
+ * secret_schema_newv: (rename-to secret_schema_new)
  * @name: the dotted name of the schema
  * @flags: the flags for the schema
  * @attribute_names_and_types: (element-type utf8 Secret.SchemaAttributeType): the attribute names and types of those attributes
@@ -176,8 +176,6 @@ G_DEFINE_BOXED_TYPE (SecretSchemaAttribute, secret_schema_attribute,
  * that are not stored by the libsecret library. Other libraries such as libgnome-keyring
  * don't store the schema name.
  *
- * Rename to: secret_schema_new
- *
  * Returns: (transfer full): the new schema, which should be unreferenced with
  *          secret_schema_unref() when done
  */
@@ -357,16 +355,12 @@ _secret_schema_ref_if_nonstatic (const SecretSchema *schema)
 void
 secret_schema_unref (SecretSchema *schema)
 {
-	gint refs;
-	gint i;
-
 	g_return_if_fail (schema != NULL);
+	/* statically-allocated or invalid SecretSchema */
+	g_return_if_fail (g_atomic_int_get (&schema->reserved) > 0);
 
-	refs = g_atomic_int_add (&schema->reserved, -1);
-	if (refs < 0) {
-		g_warning ("should not unreference a static or invalid SecretSchema");
-
-	} else if (refs == 0) {
+	if (g_atomic_int_dec_and_test (&schema->reserved)) {
+		gint i;
 		g_free ((gpointer)schema->name);
 		for (i = 0; i < G_N_ELEMENTS (schema->attributes); i++)
 			g_free ((gpointer)schema->attributes[i].name);
